@@ -10,10 +10,6 @@ use canonical::{Canon, CanonError, EncodeToVec, Sink, Source, Store};
 use canonical_derive::Canon;
 use dusk_abi::{HostModule, Query, Transaction};
 use dusk_hamt::Map;
-#[cfg(feature = "persistence")]
-use microkelvin::{
-    BackendCtor, Compound, DiskBackend, PersistError, PersistedId, Persistence,
-};
 use tracing::{trace, trace_span};
 
 use crate::call_context::CallContext;
@@ -23,10 +19,22 @@ use crate::modules::ModuleConfig;
 use crate::modules::{compile_module, HostModules};
 use crate::{Schedule, VMError};
 
+#[cfg(feature = "persistence")]
+pub mod persist;
+
 /// State of the contracts on the network.
 #[derive(Clone, Default, Canon)]
 pub struct Contracts(Map<ContractId, Contract>);
 
+use once_cell::sync::Lazy;
+use parking_lot::RwLock;
+use std::sync::Arc;
+
+/// A thread-safe singleton used to register host modules shared across all
+/// network states and call contexts.
+pub(crate) static HOST_MODULES: Lazy<Arc<RwLock<HostModules>>> =
+    Lazy::new(|| Arc::new(RwLock::new(HostModules::default())));
+
 impl Contracts {
     /// Returns a reference to the specified contracts state.
     pub fn get_contract<'a>(
@@ -101,7 +109,6 @@ impl Contracts {
 pub struct NetworkState {
     origin: Contracts,
     head: Contracts,
-    modules: HostModules,
     module_config: ModuleConfig,
 }
 
@@ -120,7 +127,6 @@ impl Canon for NetworkState {
         Ok(Self {
             origin,
             head,
-            modules: HostModules::default(),
             module_config: ModuleConfig::new(),
         })
     }
@@ -168,33 +174,6 @@ impl NetworkState {
         self.head.get_contract_mut(contract_id)
     }
 
-    /// Returns a reference to the map of registered host modules
-    pub fn modules(&self) -> &HostModules {
-        &self.modules
-    }
-
-    #[cfg(feature = "persistence")]
-    /// Persists the origin contracts stored on the [`NetworkState`] specifying
-    /// a backend ctor function.
-    pub fn persist(
-        &self,
-        ctor: fn() -> Result<DiskBackend, PersistError>,
-    ) -> Result<PersistedId, PersistError> {
-        Persistence::persist(&BackendCtor::new(ctor), &self.head.0)
-    }
-
-    #[cfg(feature = "persistence")]
-    /// Given a [`PersistedId`] restores the [`Hamt`] which stores the contracts
-    /// of the entire blockchain state.
-    pub fn restore(mut self, id: PersistedId) -> Result<Self, PersistError> {
-        let map = Map::from_generic(&id.restore()?)?;
-
-        self.origin = Contracts(map);
-        self.head = self.origin.clone();
-
-        Ok(self)
-    }
-
     /// Deploys a contract to the `head` state, returning the address of the
     /// created contract or an error.
     pub fn deploy(
@@ -325,9 +304,9 @@ impl NetworkState {
     /// Register a host function handler.
     pub fn register_host_module<M>(&mut self, module: M)
     where
-        M: HostModule + 'static,
+        M: HostModule + 'static + Sync + Send,
     {
-        self.modules.insert(module);
+        HOST_MODULES.write().insert(module);
     }
 
     /// Gets the state of the given contract in the `head` state.
